@@ -2,8 +2,9 @@ from __future__ import annotations
 
 import dataclasses
 from abc import ABC, abstractmethod
+from collections.abc import Sequence
 from threading import Lock
-from typing import Any, Generic, Sequence, TypeVar
+from typing import Any, Generic, TypeVar
 from weakref import WeakSet
 
 from opentelemetry.metrics import (
@@ -20,8 +21,12 @@ from opentelemetry.metrics import (
     UpDownCounter,
 )
 from opentelemetry.sdk.metrics import MeterProvider as SDKMeterProvider
+from opentelemetry.trace import get_current_span
 from opentelemetry.util.types import Attributes
 
+from .tracer import _LogfireWrappedSpan  # type: ignore
+from .utils import handle_internal_errors
+
 try:
     # This only exists in opentelemetry-sdk>=1.23.0
     from opentelemetry.metrics import _Gauge
@@ -35,9 +40,9 @@ except ImportError:  # pragma: no cover
 @dataclasses.dataclass
 class ProxyMeterProvider(MeterProvider):
     provider: MeterProvider
-    meters: WeakSet[_ProxyMeter] = dataclasses.field(default_factory=WeakSet)
+    meters: WeakSet[_ProxyMeter] = dataclasses.field(default_factory=WeakSet)  # type: ignore[reportUnknownVariableType]
     lock: Lock = dataclasses.field(default_factory=Lock)
-    suppressed_scopes: set[str] = dataclasses.field(default_factory=set)
+    suppressed_scopes: set[str] = dataclasses.field(default_factory=set)  # type: ignore[reportUnknownVariableType]
 
     def get_meter(
         self,
@@ -74,7 +79,7 @@ class ProxyMeterProvider(MeterProvider):
             if isinstance(self.provider, SDKMeterProvider):
                 self.provider.shutdown(timeout_millis)
 
-    def force_flush(self, timeout_millis: float = 30_000) -> None:  # pragma: no cover
+    def force_flush(self, timeout_millis: float = 30_000) -> None:
         with self.lock:
             if isinstance(self.provider, SDKMeterProvider):  # pragma: no branch
                 self.provider.force_flush(timeout_millis)
@@ -108,16 +113,19 @@ class _ProxyMeter(Meter):
             for instrument in self._instruments:
                 instrument.on_meter_set(real_meter)
 
+    def _add_proxy_instrument(self, instrument_type: type[_ProxyInstrument[InstrumentT]], **kwargs: Any) -> InstrumentT:
+        with self._lock:
+            proxy = instrument_type(self._meter, **kwargs)
+            self._instruments.add(proxy)
+            return proxy  # type: ignore
+
     def create_counter(
         self,
         name: str,
         unit: str = '',
         description: str = '',
     ) -> Counter:
-        with self._lock:
-            proxy = _ProxyCounter(self._meter.create_counter(name, unit, description), name, unit, description)
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(_ProxyCounter, name=name, unit=unit, description=description)
 
     def create_up_down_counter(
         self,
@@ -125,12 +133,7 @@ class _ProxyMeter(Meter):
         unit: str = '',
         description: str = '',
     ) -> UpDownCounter:
-        with self._lock:
-            proxy = _ProxyUpDownCounter(
-                self._meter.create_up_down_counter(name, unit, description), name, unit, description
-            )
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(_ProxyUpDownCounter, name=name, unit=unit, description=description)
 
     def create_observable_counter(
         self,
@@ -139,16 +142,9 @@ class _ProxyMeter(Meter):
         unit: str = '',
         description: str = '',
     ) -> ObservableCounter:
-        with self._lock:
-            proxy = _ProxyObservableCounter(
-                self._meter.create_observable_counter(name, callbacks, unit, description),
-                name,
-                callbacks,
-                unit,
-                description,
-            )
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(
+            _ProxyObservableCounter, name=name, unit=unit, description=description, callbacks=callbacks
+        )
 
     def create_histogram(
         self,
@@ -157,12 +153,7 @@ class _ProxyMeter(Meter):
         description: str = '',
         **kwargs: Any,
     ) -> Histogram:
-        with self._lock:
-            proxy = _ProxyHistogram(
-                self._meter.create_histogram(name, unit, description, **kwargs), name, unit, description
-            )
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(_ProxyHistogram, name=name, unit=unit, description=description, **kwargs)
 
     def create_gauge(
         self,
@@ -177,10 +168,7 @@ class _ProxyMeter(Meter):
                 'You should upgrade to 1.23.0 or newer:\n'
                 '   pip install opentelemetry-sdk>=1.23.0'
             )
-        with self._lock:
-            proxy = _ProxyGauge(self._meter.create_gauge(name, unit, description), name, unit, description)
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(_ProxyGauge, name=name, unit=unit, description=description)
 
     def create_observable_gauge(
         self,
@@ -189,16 +177,9 @@ class _ProxyMeter(Meter):
         unit: str = '',
         description: str = '',
     ) -> ObservableGauge:
-        with self._lock:
-            proxy = _ProxyObservableGauge(
-                self._meter.create_observable_gauge(name, callbacks, unit, description),
-                name,
-                callbacks,
-                unit,
-                description,
-            )
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(
+            _ProxyObservableGauge, name=name, unit=unit, description=description, callbacks=callbacks
+        )
 
     def create_observable_up_down_counter(
         self,
@@ -207,33 +188,18 @@ class _ProxyMeter(Meter):
         unit: str = '',
         description: str = '',
     ) -> ObservableUpDownCounter:
-        with self._lock:
-            proxy = _ProxyObservableUpDownCounter(
-                self._meter.create_observable_up_down_counter(name, callbacks, unit, description),
-                name,
-                callbacks,
-                unit,
-                description,
-            )
-            self._instruments.add(proxy)
-            return proxy
+        return self._add_proxy_instrument(
+            _ProxyObservableUpDownCounter, name=name, unit=unit, description=description, callbacks=callbacks
+        )
 
 
 InstrumentT = TypeVar('InstrumentT', bound=Instrument)
 
 
 class _ProxyInstrument(ABC, Generic[InstrumentT]):
-    def __init__(
-        self,
-        instrument: InstrumentT,
-        name: str,
-        unit: str,
-        description: str,
-    ) -> None:
-        self._name = name
-        self._unit = unit
-        self._description = description
-        self._instrument = instrument
+    def __init__(self, meter: Meter, **kwargs: Any) -> None:
+        self._kwargs = kwargs
+        self._instrument = self._create_real_instrument(meter)
 
     def on_meter_set(self, meter: Meter) -> None:
         """Called when a real meter is set on the creating _ProxyMeter."""
@@ -246,18 +212,11 @@ class _ProxyInstrument(ABC, Generic[InstrumentT]):
     def _create_real_instrument(self, meter: Meter) -> InstrumentT:
         """Create an instance of the real instrument. Implement this."""
 
-
-class _ProxyAsynchronousInstrument(_ProxyInstrument[InstrumentT], ABC):
-    def __init__(
-        self,
-        instrument: InstrumentT,
-        name: str,
-        callbacks: Sequence[CallbackT] | None,
-        unit: str,
-        description: str,
-    ) -> None:
-        super().__init__(instrument, name, unit, description)
-        self._callbacks = callbacks
+    @handle_internal_errors
+    def _increment_span_metric(self, amount: float, attributes: Attributes | None = None):
+        span = get_current_span()
+        if isinstance(span, _LogfireWrappedSpan):
+            span.increment_metric(self._kwargs['name'], attributes or {}, amount)
 
 
 class _ProxyCounter(_ProxyInstrument[Counter], Counter):
@@ -270,10 +229,11 @@ class _ProxyCounter(_ProxyInstrument[Counter], Counter):
         *args: Any,
         **kwargs: Any,
     ) -> None:
+        self._increment_span_metric(amount, attributes)
         self._instrument.add(amount, attributes, *args, **kwargs)
 
     def _create_real_instrument(self, meter: Meter) -> Counter:
-        return meter.create_counter(self._name, self._unit, self._description)
+        return meter.create_counter(**self._kwargs)
 
 
 class _ProxyHistogram(_ProxyInstrument[Histogram], Histogram):
@@ -284,31 +244,26 @@ class _ProxyHistogram(_ProxyInstrument[Histogram], Histogram):
         *args: Any,
         **kwargs: Any,
     ) -> None:
+        self._increment_span_metric(amount, attributes)
         self._instrument.record(amount, attributes, *args, **kwargs)
 
     def _create_real_instrument(self, meter: Meter) -> Histogram:
-        return meter.create_histogram(self._name, self._unit, self._description)
+        return meter.create_histogram(**self._kwargs)
 
 
-class _ProxyObservableCounter(_ProxyAsynchronousInstrument[ObservableCounter], ObservableCounter):
-    def _create_real_instrument(self, meter: Meter) -> ObservableCounter:  # pragma: no cover
-        return meter.create_observable_counter(self._name, self._callbacks, self._unit, self._description)
+class _ProxyObservableCounter(_ProxyInstrument[ObservableCounter], ObservableCounter):
+    def _create_real_instrument(self, meter: Meter) -> ObservableCounter:
+        return meter.create_observable_counter(**self._kwargs)
 
 
-class _ProxyObservableGauge(
-    _ProxyAsynchronousInstrument[ObservableGauge],
-    ObservableGauge,
-):
-    def _create_real_instrument(self, meter: Meter) -> ObservableGauge:  # pragma: no cover
-        return meter.create_observable_gauge(self._name, self._callbacks, self._unit, self._description)
+class _ProxyObservableGauge(_ProxyInstrument[ObservableGauge], ObservableGauge):
+    def _create_real_instrument(self, meter: Meter) -> ObservableGauge:
+        return meter.create_observable_gauge(**self._kwargs)
 
 
-class _ProxyObservableUpDownCounter(
-    _ProxyAsynchronousInstrument[ObservableUpDownCounter],
-    ObservableUpDownCounter,
-):
-    def _create_real_instrument(self, meter: Meter) -> ObservableUpDownCounter:  # pragma: no cover
-        return meter.create_observable_up_down_counter(self._name, self._callbacks, self._unit, self._description)
+class _ProxyObservableUpDownCounter(_ProxyInstrument[ObservableUpDownCounter], ObservableUpDownCounter):
+    def _create_real_instrument(self, meter: Meter) -> ObservableUpDownCounter:
+        return meter.create_observable_up_down_counter(**self._kwargs)
 
 
 class _ProxyUpDownCounter(_ProxyInstrument[UpDownCounter], UpDownCounter):
@@ -322,7 +277,7 @@ class _ProxyUpDownCounter(_ProxyInstrument[UpDownCounter], UpDownCounter):
         self._instrument.add(amount, attributes, *args, **kwargs)
 
     def _create_real_instrument(self, meter: Meter) -> UpDownCounter:
-        return meter.create_up_down_counter(self._name, self._unit, self._description)
+        return meter.create_up_down_counter(**self._kwargs)
 
 
 if Gauge is not None:  # pragma: no branch
@@ -334,10 +289,10 @@ if Gauge is not None:  # pragma: no branch
             attributes: Attributes | None = None,
             *args: Any,
             **kwargs: Any,
-        ) -> None:  # pragma: no cover
+        ) -> None:
             self._instrument.set(amount, attributes, *args, **kwargs)
 
-        def _create_real_instrument(self, meter: Meter):  # pragma: no cover
-            return meter.create_gauge(self._name, self._unit, self._description)
+        def _create_real_instrument(self, meter: Meter):
+            return meter.create_gauge(**self._kwargs)
 else:  # pragma: no cover
     _ProxyGauge = None  # type: ignore
