@@ -1,7 +1,6 @@
 import importlib
 import logging
-import sys
-from typing import Generator, Iterator
+from collections.abc import Generator, Iterator
 from unittest import mock
 
 import pytest
@@ -17,9 +16,6 @@ import logfire
 import logfire._internal.integrations.celery
 from logfire.testing import TestExporter
 
-# TODO find a better solution
-pytestmark = pytest.mark.skipif(sys.version_info < (3, 9), reason='Redis testcontainers has problems in 3.8')
-
 
 def test_missing_opentelemetry_dependency() -> None:
     with mock.patch.dict('sys.modules', {'opentelemetry.instrumentation.celery': None}):
