@@ -4,6 +4,7 @@ import importlib
 from unittest import mock
 
 import pytest
+from dirty_equals import IsFloat, IsInt
 from flask import Flask
 from inline_snapshot import snapshot
 from opentelemetry.propagate import inject
@@ -33,7 +34,7 @@ def test_wsgi_middleware(exporter: TestExporter) -> None:
         assert response.status_code == 200
         assert response.text == 'middleware test'
 
-    assert exporter.exported_spans_as_dict() == snapshot(
+    assert exporter.exported_spans_as_dict(parse_json_attributes=True) == snapshot(
         [
             {
                 'name': 'inside request handler',
@@ -91,6 +92,40 @@ def test_wsgi_middleware(exporter: TestExporter) -> None:
                     'logfire.msg_template': 'outside request handler',
                     'logfire.msg': 'outside request handler',
                     'logfire.span_type': 'span',
+                    'logfire.metrics': {
+                        'http.server.duration': {
+                            'details': [
+                                {
+                                    'attributes': {
+                                        'http.flavor': '1.1',
+                                        'http.host': 'localhost',
+                                        'http.method': 'GET',
+                                        'http.scheme': 'http',
+                                        'http.server_name': 'localhost',
+                                        'http.status_code': 200,
+                                        'net.host.name': 'localhost',
+                                        'net.host.port': 80,
+                                    },
+                                    'total': IsInt(),
+                                }
+                            ],
+                            'total': IsInt(),
+                        },
+                        'http.server.request.duration': {
+                            'details': [
+                                {
+                                    'attributes': {
+                                        'http.request.method': 'GET',
+                                        'http.response.status_code': 200,
+                                        'network.protocol.version': '1.1',
+                                        'url.scheme': 'http',
+                                    },
+                                    'total': IsFloat(),
+                                }
+                            ],
+                            'total': IsFloat(),
+                        },
+                    },
                 },
             },
         ]
