@@ -1,9 +1,13 @@
 # pyright: reportPrivateUsage=false
 from __future__ import annotations
 
+import decimal
+import enum
 import io
 import sys
+from datetime import datetime
 from typing import Any
+from unittest import mock
 
 import pytest
 from dirty_equals import IsStr
@@ -914,3 +918,66 @@ def test_console_otel_logs(capsys: pytest.CaptureFixture[str]):
             "{'key': 'value'}",
         ]
     )
+
+
+def test_truncated_json(capsys: pytest.CaptureFixture[str]) -> None:
+    with mock.patch.dict('os.environ', {'OTEL_ATTRIBUTE_VALUE_LENGTH_LIMIT': '70'}):
+        logfire.configure(
+            send_to_logfire=False,
+            console=ConsoleOptions(verbose=True, colors='never', include_timestamps=False),
+        )
+
+        logfire.info('hi', x=[1] * 100)
+
+        assert capsys.readouterr().out.splitlines() == snapshot(
+            [
+                'hi',
+                IsStr(),
+                "│ x='[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1'",
+            ]
+        )
+
+
+def test_other_json_schema_types(capsys: pytest.CaptureFixture[str]) -> None:
+    logfire.configure(
+        send_to_logfire=False,
+        console=ConsoleOptions(verbose=True, colors='never', include_timestamps=False),
+    )
+
+    class MyEnum(enum.Enum):
+        """Enum with string values."""
+
+        ABC = 'abc'
+
+    class MyStrEnum(str, enum.Enum):
+        """String-based Enum."""
+
+        STR = 'str_val'
+
+    class MyIntEnum(int, enum.Enum):
+        """Integer-based Enum."""
+
+        INT = 1
+
+    logfire.info(
+        'hi',
+        d=datetime(2020, 12, 31, 12, 34, 56),
+        x=None,
+        v=decimal.Decimal('1.0'),
+        e=MyEnum.ABC,
+        se=MyStrEnum.STR,
+        ie=MyIntEnum.INT,
+    )
+
+    assert capsys.readouterr().out.splitlines() == snapshot(
+        [
+            'hi',
+            IsStr(),
+            '│ d=datetime.datetime(2020, 12, 31, 12, 34, 56)',
+            '│ x=None',
+            "│ v=Decimal('1.0')",
+            "│ e=MyEnum('abc')",
+            "│ se=MyStrEnum('str_val')",
+            '│ ie=MyIntEnum(1)',
+        ]
+    )
